/*
    JustIce, the class file verifier for use with BCEL was written
    in 2001 by Enver Haase; (c) 2001 Enver Haase.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package de.fub.bytecode.verifier;

import java.util.ArrayList;

/**
 * A PassVerifier actually verifies a class file; it is instantiated
 * by a Verifier.
 * The verification should conform with a certain pass as described
 * in The Java Virtual Machine Specification, 2nd edition.
 * This book describes four passes. Pass one means loading the
 * class and verifying a few static constraints. Pass two actually
 * verifies some other constraints that could enforce loading in
 * referenced class files. Pass three is the first pass that actually
 * checks constraints in the code array of a method in the class file;
 * it has two parts with the first verifying static constraints and
 * the second part verifying structural constraints (where a data flow
 * analysis is used for). The fourth pass, finally, performs checks
 * that can only be done at run-time.
 * JustIce does not have a run-time pass, but certain constraints that
 * are usually delayed until run-time for performance reasons are also
 * checked during the second part of pass three.
 * PassVerifier instances perform caching.
 * That means, if you really want a new verification run of a certain
 * pass you must use a new instance of a given PassVerifier.
 *
 * @version $Id: PassVerifier.java,v 1.13 2001/08/30 08:45:41 ehaase Exp $
 * @author <A HREF="http://www.inf.fu-berlin.de/~ehaase"/>Enver Haase</A>
 * @see de.fub.bytecode.verifier.Verifier
 * @see #verify()
 */
public abstract class PassVerifier{

	/** The (warning) messages. */
	private ArrayList messages = new ArrayList(); //Type of elements: String
	
	/** The VerificationResult cache. */
	private VerificationResult verificationResult = null;

	/**
	 * This method runs a verification pass conforming to the
	 * Java Virtual Machine Specification, 2nd edition, on a
	 * class file.
	 * PassVerifier instances perform caching;
	 * i.e. if the verify() method once determined a VerificationResult,
	 * then this result may be returned after every invocation of this
	 * method instead of running the verification pass anew; likewise with
	 * the result of getMessages().
	 *
	 * @see #getMessages()
	 * @see #addMessage(String)
	 */
	public VerificationResult verify(){
		if (verificationResult == null){
			verificationResult = do_verify();
		}
		return verificationResult;
	}

	/** Does the real verification work, uncached. */
	public abstract VerificationResult do_verify();

	/**
	 * This method adds a (warning) message to the message pool of this
	 * PassVerifier. This method is normally only internally used by
	 * BCEL's class file verifier "JustIce" and should not be used from
	 * the outside. 
	 *
	 * @see #getMessages()
	 */
	public void addMessage(String message){
		messages.add(message);
	}

	/**
	 * Returns the (warning) messages that this PassVerifier accumulated
	 * during its do_verify()ing work.
	 *
	 * @see #addMessage(String)
	 * @see #do_verify()
	 */
	public String[] getMessages(){
		verify(); // create messages if not already done (cached!)
		String[] ret = new String[messages.size()];
		for (int i=0; i<messages.size(); i++){
			ret[i] = (String) messages.get(i);
		}
		return ret;
	}
}
