/*
    JustIce, the class file verifier for use with BCEL was written
    in 2001 by Enver Haase; (c) 2001 Enver Haase.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package de.fub.bytecode.verifier;

import de.fub.bytecode.*;
import de.fub.bytecode.classfile.*;
import de.fub.bytecode.generic.*;
import de.fub.bytecode.util.*;
import de.fub.bytecode.verifier.statics.*;
import de.fub.bytecode.verifier.structurals.*;
import de.fub.bytecode.verifier.exc.*;
import de.fub.bytecode.verifier.exc.Utility; // Ambigous if not declared explicitely.
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

/**
 * A Verifier instance is there to verify a class file according to The Java Virtual
 * Machine Specification, 2nd Edition.
 *
 * Pass-3b-verification includes pass-3a-verification;
 * pass-3a-verification includes pass-2-verification;
 * pass-2-verification includes pass-1-verification.
 *
 * A Verifier creates PassVerifier instances to perform the actual verification.
 *
 * @version $Id: Verifier.java,v 1.48 2001/09/12 15:04:32 ehaase Exp $
 * @author <A HREF="http://www.inf.fu-berlin.de/~ehaase"/>Enver Haase</A>
 * @see de.fub.bytecode.verifier.PassVerifier
 */
public class Verifier{
	/**
	 * The name of the class this verifier operates on.
	 */
	private final String classname;

	/** A Pass1Verifier for this Verifier instance. */
	private Pass1Verifier p1v;
	/** A Pass2Verifier for this Verifier instance. */
	private Pass2Verifier p2v;
	/** The Pass3aVerifiers for this Verifier instance. Key: Interned string specifying the method number. */
	private HashMap p3avs = new HashMap();
	/** The Pass3bVerifiers for this Verifier instance. Key: Interned string specifying the method number. */
	private HashMap p3bvs = new HashMap();

	/** Returns the VerificationResult for the given pass. */
	public VerificationResult doPass1(){
		if (p1v == null){
			p1v = new Pass1Verifier(this);
		}
		return p1v.verify();
	}

	/** Returns the VerificationResult for the given pass. */
	public VerificationResult doPass2(){
		if (p2v == null){
			p2v = new Pass2Verifier(this);
		}
		return p2v.verify();
	}
		
	/** Returns the VerificationResult for the given pass. */
	public VerificationResult doPass3a(int method_no){
		String key = Integer.toString(method_no);
		Pass3aVerifier p3av;
		p3av = (Pass3aVerifier) (p3avs.get(key));
		if (p3avs.get(key) == null){
			p3av = new Pass3aVerifier(this, method_no);
			p3avs.put(key, p3av);
		}
		return p3av.verify();
	}

	/** Returns the VerificationResult for the given pass. */
	public VerificationResult doPass3b(int method_no){
		String key = Integer.toString(method_no);
		Pass3bVerifier p3bv;
		p3bv = (Pass3bVerifier) (p3bvs.get(key));
		if (p3bvs.get(key) == null){
			p3bv = new Pass3bVerifier(this, method_no);
			p3bvs.put(key, p3bv);
		}
		return p3bv.verify();
	}
	
	/**
	 * This class may not be no-args instantiated.
	 */
	private Verifier(){
		classname = ""; // never executed anyway, make compiler happy.
	}// not noargs-instantiable

	/**
	 * Instantiation is done by the VerifierFactory.
	 *
	 * @see VerifierFactory
	 */
	Verifier(String fully_qualified_classname){
		classname = fully_qualified_classname;
		flush();
	}

	/**
	 * Returns the name of the class this verifier operates on.
	 * This is particularly interesting when this verifier was created
	 * recursively by another Verifier and you got a reference to this
	 * Verifier by the getVerifiers() method of the VerifierFactory.
	 * @see VerifierFactory
	 */
	public final String getClassName(){
		return classname;
	}

	/**
	 * Forget everything known about the class file; that means, really
	 * start a new verification of a possibly different class file from
	 * BCEL's repository.
	 *
	 */
	public void flush(){
		p1v = null;
		p2v = null;
		p3avs.clear();
		p3bvs.clear();
	}

	/**
	 * This returns all the (warning) messages collected during verification.
	 * A prefix shows from which verifying pass a message originates.
	 */
	public String[] getMessages(){
		ArrayList messages = new ArrayList();

		if (p1v != null){
			String[] p1m = p1v.getMessages();
			for (int i=0; i<p1m.length; i++){
				messages.add("Pass 1: "+p1m[i]);
			}
		}
		if (p2v != null){
			String[] p2m = p2v.getMessages();
			for (int i=0; i<p2m.length; i++){
				messages.add("Pass 2: "+p2m[i]);
			}
		}
		Iterator p3as = p3avs.values().iterator();
		while (p3as.hasNext()){
			Pass3aVerifier pv = (Pass3aVerifier) p3as.next();
			String[] p3am = pv.getMessages();
			int meth = pv.getMethodNo();
			for (int i=0; i<p3am.length; i++){
				messages.add("Pass 3a, method "+meth+" ('"+Repository.lookupClass(classname).getMethods()[meth]+"'): "+p3am[i]);
			}
		}
		Iterator p3bs = p3bvs.values().iterator();
		while (p3bs.hasNext()){
			Pass3bVerifier pv = (Pass3bVerifier) p3bs.next();
			String[] p3bm = pv.getMessages();
			int meth = pv.getMethodNo();
			for (int i=0; i<p3bm.length; i++){
				messages.add("Pass 3b, method "+meth+" ('"+Repository.lookupClass(classname).getMethods()[meth]+"'): "+p3bm[i]);
			}
		}

		String[] ret = new String[messages.size()];
		for (int i=0; i< messages.size(); i++){
			ret[i] = (String) messages.get(i);
		}
		
		return ret;
	}

	/**
	 * Verifies class files.
	 * This is a simple demonstration of how the API of BCEL's
	 * class file verifier "JustIce" may be used.
	 * You should supply command-line arguments which are
	 * fully qualified namea of the classes to verify. These class files
	 * must be somewhere in your CLASSPATH (refer to Sun's
	 * documentation for questions about this) or you must have put the classes
	 * into the BCEL Repository yourself (via 'addClass(JavaClass)').
	 */
	public static void main(String [] args){
		System.out.println("JustIce by Enver Haase, (C) 2001. http://bcel.sourceforge.net\n");
	  for(int k=0; k < args.length; k++) {

			if (args[k].endsWith(".class")){
				int dotclasspos = args[k].lastIndexOf(".class");
				if (dotclasspos != -1) args[k] = args[k].substring(0,dotclasspos);
			}
		
			args[k] = args[k].replace('/', '.');
		
			System.out.println("Now verifiying: "+args[k]+"\n");

			Verifier v = VerifierFactory.getVerifier(args[k]);
			VerificationResult vr;
		
			vr = v.doPass1();
			System.out.println("Pass 1:\n"+vr);

			vr = v.doPass2();
			System.out.println("Pass 2:\n"+vr);

			if (vr == VerificationResult.VR_OK){
				JavaClass jc = Repository.lookupClass(args[k]);
				for (int i=0; i<jc.getMethods().length; i++){
					vr = v.doPass3a(i);
					System.out.println("Pass 3a, method "+i+" ['"+jc.getMethods()[i]+"']:\n"+vr);

					vr = v.doPass3b(i);
					System.out.println("Pass 3b, method number "+i+" ['"+jc.getMethods()[i]+"']:\n"+vr);
				}
			}
		
			System.out.println("Warnings:");
			String[] warnings = v.getMessages();
			if (warnings.length == 0) System.out.println("<none>");
			for (int j=0; j<warnings.length; j++){
				System.out.println(warnings[j]);
			}

			System.out.println("\n");
	  
			// avoid swapping.
	  	v.flush();
	  	Repository.clearCache();
			System.gc();
	  }
	}
}
