/*
    JustIce, the class file verifier for use with BCEL was written
    in 2001 by Enver Haase; (c) 2001 Enver Haase.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package de.fub.bytecode.verifier;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

/**
 * This class produces instances of the Verifier class. Its purpose is to make
 * sure that they are singleton instances with respect to the class name they
 * operate on. That means, for every class (represented by a unique fully qualified
 * class name) there is exactly one Verifier.
 *
 * @version $Id: VerifierFactory.java,v 1.15 2001/09/12 12:56:13 ehaase Exp $
 * @author <A HREF="http://www.inf.fu-berlin.de/~ehaase"/>Enver Haase</A>
 */
public class VerifierFactory{

	/**
	 * The HashMap that holds the data about the already-constructed Verifier instances.
	 */
	private static HashMap hashMap = new HashMap();

	/**
	 * The VerifierFactoryObserver instances that observe the VerifierFactory.
	 */
	private static Vector observers;

	/**
	 * The VerifierFactory is not instantiable.
	 */
	private VerifierFactory(){}
	
	/**
	 * Returns the (only) verifier responsible for the class with the given name.
	 * Possibly a new Verifier object is transparently created.
	 * @return the (only) verifier responsible for the class with the given name.
	 */
	public static Verifier getVerifier(String fully_qualified_classname){
		fully_qualified_classname = fully_qualified_classname;
		
		Verifier v = (Verifier) (hashMap.get(fully_qualified_classname));
		if (v==null){
			v = new Verifier(fully_qualified_classname);
			hashMap.put(fully_qualified_classname, v);
			// notify the observers
			if (observers != null){
				Iterator i = observers.iterator();
				while (i.hasNext()){
					VerifierFactoryObserver vfo = (VerifierFactoryObserver) i.next();
					vfo.notify(fully_qualified_classname);
				}
			}
		}
		
		return v;
	}

	/**
	 * Returns all Verifier instances created so far.
	 * This is useful when a Verifier recursively lets
	 * the VerifierFactory create other Verifier instances
	 * and if you want to verify the transitive hull of
	 * referenced class files.
	 */
	public static Verifier[] getVerifiers(){
		Verifier[] vs = new Verifier[hashMap.values().size()];
		return (Verifier[]) (hashMap.values().toArray(vs));	// Because vs is big enough, vs is used to store the values into and returned!
	}

	/**
	 * Adds the VerifierFactoryObserver o to the list of observers.
	 */
	public static void addObserver(VerifierFactoryObserver o){
		if (observers == null){
			observers = new Vector();
		}
		observers.addElement(o);
	}
	
	/**
	 * Removes the VerifierFactoryObserver o from the list of observers.
	 */
	public static void removeObserver(VerifierFactoryObserver o){
		if (observers != null){
			observers.removeElement(o);
		}
	}
}
