/*
    JustIce, the class file verifier for use with BCEL was written
    in 2001 by Enver Haase; (c) 2001 Enver Haase.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package de.fub.bytecode.verifier.statics;

import de.fub.bytecode.classfile.*;

/**
 * BCEL's Node classes (those from the classfile API that <B>accept()</B> Visitor
 * instances) have <B>toString()</B> methods that were not designed to be robust,
 * this gap is closed by this class.
 * When performing class file verification, it may be useful to output which
 * entity (e.g. a <B>Code</B> instance) is not satisfying the verifier's
 * constraints, but in this case it could be possible for the <B>toString()</B>
 * method to throw a RuntimeException.
 * A (new StringRepresentation(Node n)).toString() never throws any exception.
 * Note that this class also serves as a placeholder for more sophisticated message
 * handling in future versions of JustIce.
 * 
 * @version $Id: StringRepresentation.java,v 1.11 2001/08/30 08:45:42 ehaase Exp $
 * @author <A HREF="http://www.inf.fu-berlin.de/~ehaase"/>Enver Haase</A>
 */
public class StringRepresentation extends de.fub.bytecode.classfile.EmptyVisitor implements Visitor{
	/** The string representation, created by a visitXXX() method, output by toString(). */
	private String tostring;
	/**
	 * Creates a new StringRepresentation object which is the representation of n.
	 *
	 * @see #toString()
	 */
	public StringRepresentation(Node n){
		n.accept(this);
	}
	/**
	 * Returns the String representation.
	 */
	public String toString(){
		return tostring;
	}
	/**
	 * Returns the String representation of the Node object obj;
	 * this is obj.toString() if it does not throw any RuntimeException,
	 * or else it is a string derived only from obj's class name.
	 */
	private String toString(Node obj){
		String ret;
		try{
			ret = obj.toString();
		}
		catch(RuntimeException e){
			String s = obj.getClass().getName();
			s = s.substring(s.lastIndexOf(".")+1);
			ret = "<<"+s+">>";
		}
		return ret;
	}
	////////////////////////////////
	// Visitor methods start here //
	////////////////////////////////
	// We don't of course need to call some default implementation:
	// e.g. we could also simply output "Code" instead of a possibly
	// lengthy Code attribute's toString().
	public void visitCode(Code obj){
		//tostring = toString(obj);
		tostring = "<CODE>"; // We don't need real code outputs.
	}
	public void visitCodeException(CodeException obj){
		tostring = toString(obj);
	}
	public void visitConstantClass(ConstantClass obj){
		tostring = toString(obj);
	}
	public void visitConstantDouble(ConstantDouble obj){
		tostring = toString(obj);
	}
	public void visitConstantFieldref(ConstantFieldref obj){
		tostring = toString(obj);
	}
	public void visitConstantFloat(ConstantFloat obj){
		tostring = toString(obj);
	}
	public void visitConstantInteger(ConstantInteger obj){
		tostring = toString(obj);
	}
	public void visitConstantInterfaceMethodref(ConstantInterfaceMethodref obj){
		tostring = toString(obj);
	}
	public void visitConstantLong(ConstantLong obj){
		tostring = toString(obj);
	}
	public void visitConstantMethodref(ConstantMethodref obj){
		tostring = toString(obj);
	}
	public void visitConstantNameAndType(ConstantNameAndType obj){
		tostring = toString(obj);
	}
 	public void visitConstantPool(ConstantPool obj){
		tostring = toString(obj);
 	}
	public void visitConstantString(ConstantString obj){
		tostring = toString(obj);
	}
	public void visitConstantUtf8(ConstantUtf8 obj){
		tostring = toString(obj);
	}
	public void visitConstantValue(ConstantValue obj){
		tostring = toString(obj);
	}
	public void visitDeprecated(Deprecated obj){
		tostring = toString(obj);
	}
	public void visitExceptionTable(ExceptionTable obj){
		tostring = toString(obj);
	}
	public void visitField(Field obj){
		tostring = toString(obj);
	}
	public void visitInnerClass(InnerClass obj){
		tostring = toString(obj);
	}
	public void visitInnerClasses(InnerClasses obj){
		tostring = toString(obj);
	}
	public void visitJavaClass(JavaClass obj){
		tostring = toString(obj);
	}
	public void visitLineNumber(LineNumber obj){
		tostring = toString(obj);
	}
	public void visitLineNumberTable(LineNumberTable obj){
		tostring = "<LineNumberTable: "+toString(obj)+">";
	}
	public void visitLocalVariable(LocalVariable obj){
		tostring = toString(obj);
	}
	public void visitLocalVariableTable(LocalVariableTable obj){
		tostring = "<LocalVariableTable: "+toString(obj)+">";
	}
	public void visitMethod(Method obj){
		tostring = toString(obj);
	}
	public void visitSourceFile(SourceFile obj){
		tostring = toString(obj);
	} 
	public void visitSynthetic(Synthetic obj){
		tostring = toString(obj);
	} 
	public void visitUnknown(Unknown obj){
		tostring = toString(obj);
	}
}
